/* OneGL1/VideoGL1 Copyright (C) Roy Longbottom 2004
 *
 *  OneGL1.exe runs the benchmark either stand-alone at existing screen
 *  settings or varying settings via VideoGL1.exe. Results are saved
 *  in file VideoLog.txt. The following tests are run :
 *
 *  testPass 0 to 5 have a camera moving up and down a tunnel with
 *  moving flyers with cubes and eggs at the end. testPass 6 and 7
 *  represent a real complex kitchen design program.
 *
 *  testPass
 *
 *  0  Plain Colours few objects
 *  1  Tunnel plain colours few textured objects at end
 *  2  Mainly plain colours more objects anf flyers in tunnel
 *  3  As 1 but more textured objects
 *  4  As 3 but textured tunnel walls
 *  5  As 4 but textured tunnel floor and ceiling
 *  6  Wireframe rotating kitchen
 *  7  Rotating kitchen textured with wall, floor and worktop tiles 
 *
 *  Speed is measured in Frames Per Second
 *
 *  Modules
 *
 *  config.cpp - via WhatConfig() identifies CPU type/speed, memory
 *            size, OS version, also getStartTime() and getRunSecs() 
 *  OneGL1.rc - Only icon ID
 *  OneGL1.h, OneGL1.ico
 *  render.c - OpenGL set up and rendering - bSetupPixelFormat(),
 *       initializeFont(), initialize(), draw_scene(), deleteFont()
 *       ShowParameters(), ShowPixelFormat(). The two Show functions
 *       need showFlags = TRUE and showParam = TRUE below to output
 *       details to a log file.
 *
 *  OneGL1.cpp
 *
 *  Particularly with Windows XP, on running a benchmark that displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using the testPass variable.
 *
 *  Functions
 *
 *  WinMain() - usual Windows functions, runs setUp(), runTests
 *            and endTests()
 *  InitApp() - usual Windows functions, processes command line,
 *            allocates variable values. 
 *  WindowProc() - usual Windows functions for msgs, not used except
 *            for suggested Palette functions.
 *  InitGlobals() - Initialize some global variables *  
 *  gettimedate() - clock time and date
 *  setUp() - Opens log file, bSetupPixelFormat(), selects palette if
 *            needed, creates OpenGL, initialize(), variable assignment.
 *  runTests() - For each testPass - draw_scene() to obtain set up time,
 *            draw_scene() for 1 second to calculate degrees to adjust
 *            rotation speed, draw_scene() for runTime.
 *  XMsg() - saveData() for errors
 *  __cdecl Msg2() - Error messages from render.c
 *  endTests() - Release OpenGL, saveData()
 *  saveData() - Save data, error messages and results
 *  CleanUpAndPostQuit() - Release OpenGL and Exit
 *
 *  Run Time objects
 *
 *  OpenGL.bmp   - 64 x 64 pixels 16 colours OpenGL logo for flyers 
 *  aTile256.bmp - 256 x 256 pixels x 256 colours Texture for tunnel
 *                 floor/ceiling
 *  windows.bmp  - 256 x 256 pixels x 256 colours texture for eggs
 *  Tile1.bmp    - 256 x 256 pixels x 256 colours texture for kitchen floor
 *  Tile2.bmp    - 256 x 256 pixels x 256 colours texture for kitchen worktops
 *  Tile3.bmp    - 256 x 256 pixels x 256 colours texture for kitchen borders
 *  Tile4.bmp    - 256 x 256 pixels x 256 colours texture for kitchen walls
 *
 *   ************************* WARNING ****************************** 
 *  glu32.lib and opengl32.lib MUST BE INCLUDED in compiler Link Library
 *  modules.
 *
 */
 
#define NAME "OGLTest"
#define TITLE " Roy Longbottom's OpenGL Function Timing Test "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <direct.h>

#include "OneGL1.h"

struct _myglobs
{
    int BPP;                   // bit depth of current display
    int Wide;                  // width current display
    int High;                  // height current display
    BOOL bQuit         ;  
} myglobs;

//  ************** PROTOPTYPES *****************

static void InitGlobals(void);
static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
static BOOL saveData();
static BOOL runTests();
static BOOL XMsg(LPSTR);
void gettimedate();
void __cdecl Msg2( LPSTR Mess, GLenum glError);
static BOOL setUp();
static BOOL endTests();

#define version "Version 2.4"
TEXTMETRIC tmt;
HANDLE    MyInstance;
HWND   mainHWND;
HGLRC  hRC = NULL;
SYSTEMTIME sTime;
FILE    *outfile;

char    timedate[40];
char    commandLine[256] = "";
char    testTitle[10][80] = {" "};
char    heading1[10][8];
char    heading2[10][8];
char    mbuff[256];
char    resultData2[200];
char    logfile[100] = "VideoLog.txt";
char    bmpfile[100] = "atile256.bmp";

int    testx;
int    testPhase = 0;
int    testsDone = 0;
int    screenWide;
int    screenHigh;
int    bitsaPixel;
int    paramRes;
int    delayTime = 1;
int    runTime = 5;
int    stx = 2;
int    singleTest = 2;
int    minColor = 1;
int    maxColor = 4;
int    oneTest  = 0;
int    moveFast = 0;
int    maxTests = 10;
int    lastTest =  8;
int    lastOne  = 8;
int    firstTest = 0;
int    sceneToDo = 0;
int    maxRes = 9;
int    sceneIf = 0;
int    firstToSave = 0;
int    linesT6;
int    testPass;
int    frameCount;
int    wideUsed[40];
int    highUsed[40];
int    bitsUsed[40];
int    resultsToSave = 0;

short  keyPress;
short  keyPress1;
short  keyUp;
short  keyDown;
short  keyLeft;
short  keyRight;

long   setUpTicks = 0;
long   buildTicks = 0;

float  aspect;
float  foreBack = 0;
float  leftRight =0;
float  degrees;

double  result1;
double  resultS[40][10];

GLenum  glErr;

 
BOOL    useFlip = TRUE;
BOOL    autoSave = TRUE;
BOOL    autoRun = FALSE;
BOOL    autoDo  = TRUE;
BOOL    noSave = FALSE;
BOOL    saveHeadings = TRUE;
BOOL    saveResults = TRUE;
BOOL    saveError = FALSE;
BOOL    saveEnd = TRUE;
BOOL    fileError = FALSE;
BOOL    timeIt;
BOOL    timeIt2;
BOOL    showFlags = FALSE;
BOOL    showParam = FALSE;
BOOL    doSave = FALSE;
BOOL    escPressed = FALSE;


// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow )
{
    MSG         msg;
    
    /* previous instances do not exist in Win32 */
    if (hPrevInstance)
        return 0;
    
    cmdline = cmdline;


    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    testx = 0;       
    gettimedate();
    saveError = FALSE;
    testPhase = 4;
    
    if (!setUp()) CleanUpAndPostQuit();
    testPass=firstTest;
    
    while(!myglobs.bQuit)
    {
        
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (testPass == lastTest)
        {
            CleanUpAndPostQuit();
        }
        else if (testPass < lastTest)
        {
            ShowCursor(FALSE);
            if(!runTests()) // Run Tests                          
            {
                endTests();
                ShowCursor(TRUE);
                testPass = lastTest;
            }
            else
            {
                testPass = testPass + 1;
                if (testPass == lastTest) endTests();
            }
        }
        else
        {
           // make sure we go to sleep if we have
           // nothing else to do
           WaitMessage();
        }
    }
    return msg.wParam;
    
} // WinMain

 // CleanUpAndPostQuit
 
//               Initialization and object creation
// InitApp
// Creates window and initializes all objects neccessary to begin
// Rendering

static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND win;
    HDC hdc;
    WNDCLASS wc;
    int i, j;
    
   
    // set up and registers the window class
    
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  NULL;
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;
    
    // Initialize the global variables and allow the sample code
    // to override some of these default settings.

    InitGlobals();

    
    // Create the window

    win =
        CreateWindow
        ( 
            NAME,                          // class
            TITLE,                         // caption 
            WS_POPUP | WS_MAXIMIZE |
                        WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
            0,                             // init. x pos
            0,                             // init. y pos 
            GetSystemMetrics(SM_CXSCREEN), // init. x size
            GetSystemMetrics(SM_CYSCREEN), // init. y size
            NULL,                          // parent window
            NULL,                          // menu handle
            hInstance,                     // program handle
            NULL                           // create parms
        );
    if (!win)
        return FALSE;

    mainHWND = win;

    strcpy(commandLine, strupr(cmdline));    

    
    // Record the current display BPP and size

    hdc = GetDC(win);

    myglobs.BPP = GetDeviceCaps(hdc, BITSPIXEL);
    myglobs.Wide = GetDeviceCaps(hdc, HORZRES);    
    myglobs.High = GetDeviceCaps(hdc, VERTRES);

    ReleaseDC(win, hdc);
    
    bitsaPixel = myglobs.BPP;
    screenWide = myglobs.Wide;
    screenHigh = myglobs.High;

    minColor = 0;
    if (bitsaPixel == 16) minColor = 1; 
    if (bitsaPixel == 24) minColor = 2;
    if (bitsaPixel == 32) minColor = 3;
    maxColor = minColor + 1; 
    
    gettimedate();

    testPhase = 1;

     
    // command line scan

    if (strlen(commandLine))
    {
       paramRes = 0;
       for (j=0; j<lstrlen(commandLine); j++)
       {
          if (strnicmp(&commandLine[j],  "RUN", 3) == 0)
          {
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "NOFLIP", 6) == 0)
          {
              useFlip = FALSE;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "RUNSECS ", 8) == 0)
          {
              sscanf(&commandLine[j+8], "%d", &runTime);
              if (runTime < 1) runTime = 1;
              if (runTime > 900) runTime = 900;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "DELAY ", 6) == 0)
          {
              sscanf(&commandLine[j+6], "%d", &delayTime);
              if (delayTime < 1) delayTime = 1;
              if (delayTime > 10) delayTime = 10;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "MINSECS", 7) == 0)
          {
              runTime = 1;
              delayTime = 1;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "MAXSECS", 7) == 0)
          {
              runTime = 900;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T1", 2) == 0)
          {
              singleTest = 1;
              oneTest = 0;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T2", 2) == 0)
          {
              singleTest = 1;
              oneTest = 1;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T3", 2) == 0)
          {
              singleTest = 1;
              oneTest = 2;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T4", 2) == 0)
          {
              singleTest = 1;
              paramRes = 1;
              oneTest = 3;
          }
          if (strnicmp(&commandLine[j],  "T5", 2) == 0)
          {
              singleTest = 1;
              oneTest = 4;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T6", 2) == 0)
          {
              singleTest = 1;
              oneTest = 5;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T7", 2) == 0)
          {
              singleTest = 1;
              oneTest = 6;
              paramRes = 1;
          }
          if (strnicmp(&commandLine[j],  "T8", 2) == 0)
          {
              singleTest = 1;
              oneTest = 7;
              paramRes = 1;
          }

          if (strnicmp(&commandLine[j],  "NOSAVE", 6) == 0)
          {
              autoSave = FALSE;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "NORUN", 5) == 0)
          {
              autoDo = FALSE;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SPEED/10", 8) == 0)
          {
              moveFast = 1;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SPEED/100", 9) == 0)
          {
              moveFast = 2;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SPEEDx10", 8) == 0)
          {
              moveFast = 3;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SAVEHR", 6) == 0)
          {
              saveHeadings = TRUE;
              saveResults = TRUE;
              saveEnd = FALSE;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SAVER", 5) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = FALSE;
              paramRes = 1; 
          }
          if (strnicmp(&commandLine[j],  "SAVERE", 6) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = TRUE;
              paramRes = 1; 
          }
       }
    }    // command line scan


    if (paramRes > 0)
    {
        if (autoDo) autoRun = TRUE;
        if (autoSave) noSave = FALSE;  // auto save
    }

    for(i = 0; i<maxTests; i++)
    {
        strcpy(testTitle[i], " ");
    }

    lastTest = lastOne;
    firstTest = 0;
    
    strcpy(testTitle[0], "1 Single Plain          ");
    strcpy(testTitle[1], "2 Single Texture        ");
    strcpy(testTitle[2], "3 Multiple Plain        ");
    strcpy(testTitle[3], "4 Multiple Texture      ");
    strcpy(testTitle[4], "5 More Textures         ");
    strcpy(testTitle[5], "6 Even More Textures    ");
    strcpy(testTitle[6], "7 Wireframe Kitchen     ");
    strcpy(testTitle[7], "8 Tiled Kitchen         ");
       
    strcpy(heading1[0], " Single");
    strcpy(heading2[0], "  Plain");
    strcpy(heading1[1], " Single");
    strcpy(heading2[1], "Texture");
    strcpy(heading1[2], "  Multi");
    strcpy(heading2[2], "  Plain");
    strcpy(heading1[3], "  Multi");
    strcpy(heading2[3], "Texture");
    strcpy(heading1[4], "   More");
    strcpy(heading2[4], "Texture");
    strcpy(heading1[5], "  More+");
    strcpy(heading2[5], "Texture");
    strcpy(heading1[6], "   Wire");
    strcpy(heading2[6], "Kitchen");
    strcpy(heading1[7], "  Tiled");
    strcpy(heading2[7], "Kitchen");

    if (singleTest == 1)
    {    
        firstTest = oneTest;
        lastTest = firstTest + 1;
    }

    keyPress = GetAsyncKeyState(VK_ESCAPE);
    keyUp  = GetAsyncKeyState(VK_UP);
    keyDown  = GetAsyncKeyState(VK_DOWN);
    keyLeft  = GetAsyncKeyState(VK_LEFT);
    keyRight  = GetAsyncKeyState(VK_RIGHT);

    // Display the window
     
    ShowWindow(win, cmdshow);   
    UpdateWindow(win);

    if (!autoRun)
    {
        sprintf(mbuff, "Test at %d x %d x %d bits\n\n"
                      "Click Cancel then VideoGL1 icon to\n"
                      "test at multiple screen settings\n\n" 
                      "At end, results should be\n"
                      "in file %s\n\n"
                      "Press OK to run", 
                   screenWide, screenHigh, bitsaPixel, logfile);
       if (MessageBox(NULL, mbuff, "VidoGL1 OpenGL Speed Tests",
                    MB_ICONINFORMATION | MB_OKCANCEL) == IDCANCEL)
       {
           myglobs.bQuit = TRUE;
       }
    }
    return TRUE;
}  // InitApp


// WindowProc - handle messages for the main application window
 

long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
    HDC     hDC;
        
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
         break;

    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_PAINT:
        break;
           
    case WM_KEYDOWN:
        break;

    case WM_ACTIVATE:
        break;

    case WM_SIZE:
        return 0;

   case WM_PALETTECHANGED:
       if (ghPalette != NULL && (HWND) wparam != hwnd) 
       {
           hDC = GetDC(hwnd);
           UnrealizeObject( ghPalette );
           SelectPalette( hDC, ghPalette, FALSE );
           RealizePalette( hDC );
           ReleaseDC(hwnd, hDC);
           return 0;
       }
       break;

   case WM_QUERYNEWPALETTE: 
       if ( ghPalette != NULL ) 
       {
           hDC = GetDC(hwnd);
           UnrealizeObject( ghPalette );
           SelectPalette( hDC, ghPalette, FALSE );
           RealizePalette( hDC );
           ReleaseDC(hwnd, hDC);
           return TRUE;
       }
       break;

    case WM_COMMAND:
         break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc

 // InitGlobals
 // Initialize the global variables
 
void
InitGlobals(void)
{
    memset(&myglobs, 0, sizeof(myglobs));
    myglobs.bQuit = FALSE;
}
    

void
CleanUpAndPostQuit(void)
{
     if (hRC) wglDeleteContext(hRC);
     myglobs.bQuit = TRUE;
}

void __cdecl Msg2( LPSTR Mess, GLenum glError)
{
    lstrcpy(mbuff, Mess);
    
    if (glError & GL_INVALID_ENUM) strcat(mbuff,      "\n GL_INVALID_ENUM"); 
    if (glError & GL_INVALID_VALUE) strcat(mbuff,     "\n GL_INVALID_VALUE");
    if (glError & GL_INVALID_OPERATION) strcat(mbuff, "\n GL_INVALID_OPERATION");
    if (glError & GL_STACK_OVERFLOW) strcat(mbuff,    "\n GL_STACK_OVERFLOW");
    if (glError & GL_STACK_UNDERFLOW) strcat(mbuff,   "\n GL_STACK_UNDERFLOW");
    if (glError & GL_OUT_OF_MEMORY) strcat(mbuff,     "\n GL_OUT_OF_MEMORY");
    
    XMsg(mbuff);
}

static BOOL XMsg(LPSTR amessage)
{
    ShowCursor(TRUE);

    lstrcpy(mbuff, amessage);

    
    if (!fileError)
    {
        MessageBox(NULL, mbuff, "Error - Msg saved in log file",
                                       MB_ICONINFORMATION | MB_OK);
        saveError = TRUE;
        saveEnd = TRUE;
        stx = singleTest;
        singleTest = 2;
                
        saveData();               
    }
    else
    {
        MessageBox(NULL, mbuff, "Error", MB_ICONINFORMATION | MB_OK);
    }
    CleanUpAndPostQuit();
    return FALSE;
}

void gettimedate()
{
    GetLocalTime(&sTime);
    sprintf(timedate, "%d/%d/%d %d:%d:%d \n",
                      sTime.wDay, sTime.wMonth, sTime.wYear,
                      sTime.wHour, sTime.wMinute, sTime.wSecond);
    return;
}

static BOOL setUp()
{
    int i, j;
    char buf[40];
    HDC  hdc;
    int fontHeight;

    outfile = fopen(logfile,"a+");  
    if (outfile == NULL)
    {
        fileError = TRUE;
        XMsg("Open log file failed");
        return FALSE;
    
    }
    if (singleTest > 1)
    {
       if (saveHeadings)
       {             
           fprintf(outfile,
           "\n\n    *********************************"
                          "***************************************\n\n");   
           fprintf(outfile,
            "       Windows 95/NT/XP OpenGL Test 1 %s, %s",
                                                   version, timedate);
           fprintf(outfile,
           "                  Copyright (C) 1999 - 2003, Roy Longbottom\n\n");
           if (testPhase > 0)
           { 
           
               if (stx == 2)
               {
                   fprintf(outfile, "                   "
                                        "Running All Tests,"); 
               }
               else if (stx == 1)
               {
                   fprintf(outfile, "                   "
                                       "Running Single Test,");
               }
               if (useFlip)
               {
                   fprintf(outfile, " With Back Buffer\n\n");
               }
               else
               {
                   fprintf(outfile, " With Front Buffer Only\n\n");
               }
           }
                                  
           fprintf(outfile, "                 ");
           fprintf(outfile,
           " ........................ Frames Per Second ........................\n"); 
           fprintf(outfile, "   Resolution ");
           for (testPass=0; testPass<lastTest; testPass++)
           {
               fprintf (outfile, " %8s", heading1[testPass]);
           }
           fprintf (outfile, "\n");
           fprintf(outfile, "              ");
           for (testPass=0; testPass<lastTest; testPass++)
           {
               fprintf (outfile, " %8s", heading2[testPass]);
           }
           fprintf (outfile, "\n\n");
    
           saveHeadings = FALSE;           
       }
    }
    else
    {
        fprintf(outfile,
           "\n\n    *********************************"
                          "***************************************\n\n");   
        
        fprintf (outfile, "\n Single test %s ", timedate);
        fprintf (outfile, "%s \n", testTitle[firstTest]);   
        
        if (useFlip)
        {
           fprintf(outfile, " Back Buffer/Flip \n");
        }
        else
        {
           fprintf(outfile, " Front Buffer only, no Flip \n");
        }
        fprintf (outfile, " %s\n", resultData2);
        
        fprintf (outfile, " %d x %d x %d bits/pixel\n",
                                 screenWide, screenHigh, bitsaPixel);
    }
    fflush(outfile);

    strcpy(resultData2, "Speed ");                                      
    strcpy(buf, "Normal");
    if (moveFast == 1) strcpy(buf, "/10");
    if (moveFast == 2) strcpy(buf, "/100");
    if (moveFast == 3) strcpy(buf, "x10");
    strcat(resultData2, buf);
    strcat(resultData2, ", bmp file ");
    strcat(resultData2, bmpfile);
    strcat(resultData2, ", ");
    if (sceneToDo == 0) strcpy(buf, "Normal Tests");
    if (sceneToDo == 1) strcpy(buf, "Back Scene");
    if (sceneToDo == 2) strcpy(buf, "Front Tunnel");
    strcat(resultData2, buf);

    testsDone = 0;
    
    keyPress = GetAsyncKeyState(VK_ESCAPE);
    keyUp  = GetAsyncKeyState(VK_UP);
    keyDown  = GetAsyncKeyState(VK_DOWN);
    keyLeft  = GetAsyncKeyState(VK_LEFT);
    keyRight  = GetAsyncKeyState(VK_RIGHT);


    for(i=0; i<40; i++)
    {
        for(j=0; j<10; j++)
        {
            resultS[i][j] = 0.0;
        }
    }

    timeIt = TRUE;
    timeIt2 = TRUE;
    testx = 0;
       

    wideUsed[testx] = screenWide;
    highUsed[testx] = screenHigh;
    bitsUsed[testx] = bitsaPixel;
    aspect = (float)screenWide / (float)screenHigh;
    
    if (testx == 0 && timeIt)
    {
        getStartTime();
    }
    
    ShowWindow(mainHWND, SW_SHOWMAXIMIZED);    

 
    //   Render ***********************

    hdc = GetDC(mainHWND);
    if (!bSetupPixelFormat(hdc, useFlip, bitsaPixel)) return FALSE;

    if(ghPalette)
    {
       UnrealizeObject(ghPalette);
       SelectPalette(hdc, ghPalette, FALSE);
       RealizePalette(hdc);
    }
    
    if (showFlags) ShowPixelFormat(hdc);

    hRC = wglCreateContext( hdc );
    if (hRC == NULL)
    {
        XMsg("wglCreateContext failed");
        return FALSE;
    }
            
    if (!wglMakeCurrent( hdc, hRC ))
    {    
        XMsg("wglMakeCurrent failed");
        return FALSE;
    }
       
    fontHeight = 14 * screenWide / 640;
    initializeFont(hdc, fontHeight);
   
    firstToSave = 0;
    ShowCursor(FALSE);
    
    sceneIf = 0;
    if (singleTest == 1) sceneIf = sceneToDo;

    initialize(mainHWND, bmpfile);

    if (testx == 0 && timeIt)
    {
        getRunSecs();
        setUpTicks = (long)(runSecs * 1000);
        timeIt = FALSE;
        if (showParam) ShowParameters(hdc);                
    }
    ReleaseDC(mainHWND, hdc);
    linesT6 = 0;

    return TRUE;
    
} // setUp


static BOOL runTests()
{
    int calcCount;
    
                
//  Was for (testPass=firstTest; testPass<lastTest; testPass++)
    {     
        if (testx == 0 && timeIt2)
        {
            getStartTime();
        }
        frameCount = 0;    
        draw_scene(mainHWND, testPass, 1.0f, useFlip, aspect, 0, sceneIf, 0, 0, 0, 0, 0, 0);
        
        glFlush();
        glFinish();

        if (testx == 0 && timeIt2)
        {
            getRunSecs();
            buildTicks = (long)(runSecs * 1000);
            timeIt2 = FALSE;                                                      
        }
        
        // calibrate
        
        getStartTime();              

        do
        {
            draw_scene(mainHWND, testPass, 1.0f, useFlip, aspect, 0, sceneIf, 0, 0, 0, 0, 0, 0);
            glFinish();

            getRunSecs();
            frameCount = frameCount + 1;
        
        }
        while(runSecs < 1.0);

        if (testPass == 6) linesT6 = lines;

        degrees = (float) 45.0 / (float) frameCount;
        if (degrees > 5.0) degrees = 5.0;

        frameCount = 0;    
        
        glFlush();
        glFinish();
        result1 = 0;
        calcCount = 0;
        
        getStartTime();              
              
        do
        {
            draw_scene(mainHWND, testPass, degrees, useFlip, aspect,
                                moveFast, sceneIf, foreBack, leftRight,
                                screenWide, screenHigh, bitsaPixel, result1);
            glFinish();

            getRunSecs();
              
            frameCount = frameCount + 1;
            if (calcCount == 0)
            {
                result1 = (double) frameCount / runSecs;
                if (runSecs > 0.1) calcCount = (int)(result1 / 5);
            }
            else
            {
                calcCount = calcCount - 1;
            }
        
            if (GetAsyncKeyState(VK_UP) != keyUp)
            {
                foreBack = foreBack - 1;
            }
            if (GetAsyncKeyState(VK_DOWN) != keyDown)
            {
                foreBack = foreBack + 1;
            }
            if (GetAsyncKeyState(VK_LEFT) != keyLeft)
            {
                leftRight = leftRight - 1;
            }
            if (GetAsyncKeyState(VK_RIGHT) != keyRight)
            {
                leftRight = leftRight + 1;
            }         
            if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
            {
                testPass = lastTest;
                escPressed = TRUE;
                fprintf(outfile, " \n ESCAPE Pressed\n\n");
                return FALSE;
            }
        }
        while(runSecs < (double)runTime);

        resultS[testx][testPass] = (double) frameCount / runSecs;

        if (testPass < lastTest) Sleep(delayTime*1000);

        glErr = glGetError();
        if (glErr != GL_NO_ERROR)
        {
            Msg2("GL ERROR runTests end", glErr);
            return FALSE;
        } 
    }

    return TRUE; 

} // runTests

static BOOL endTests()
{
    HDC  hdc;

    ShowCursor(TRUE);

    hdc = GetDC(mainHWND);
    
    deleteFont(hdc);
    wglMakeCurrent (NULL, NULL) ;

    if (hRC) wglDeleteContext(hRC);
                
    ReleaseDC(mainHWND, hdc);
    
    doSave = TRUE;
    if (!noSave)
    {
          firstToSave = testx;
          resultsToSave = testx + 1;
          saveData();
    }                         
    testx = testx + 1;
    resultsToSave = testx;
           
    if (!escPressed) Sleep(200);          

    ShowWindow(mainHWND, SW_SHOWMAXIMIZED);

    return TRUE;

} // endTests


static BOOL saveData()
{
   int i, j;

   if (saveError)
   {
       fprintf (outfile, "\nResolution %d x %d x %d bits/pixel\n",
                             screenWide, screenHigh, bitsaPixel);
       fprintf (outfile, "ERROR MESSAGE %s \n\n", mbuff);
   }

   if (singleTest > 1)
   {

       if (saveResults)
       {
           for (i=firstToSave; i<resultsToSave; i++)
           {        
               fprintf (outfile, " %4d %4d %2d ", wideUsed[i],
                                           highUsed[i], bitsUsed[i]);
               for (j=0; j<lastTest; j++)
               {
                   fprintf (outfile,  "%9.2f", resultS[i][j]); 
               }
               fprintf (outfile, "\n");
           }
       }
       if (saveEnd)
       {
           fprintf (outfile,
              "\n First OpenGL Set Up %d ms, First Frame %d ms\n",
                                              setUpTicks, buildTicks);                                                  
           fprintf (outfile, "\n %s\n\n", resultData2);

           
           fflush (outfile);
           gettimedate();
           WhatConfig();
           fprintf(outfile, " %s\n", configData1);
           fprintf(outfile, " %s\n", configData2);
           fprintf(outfile, " %s\n", configData3);
               
           fprintf (outfile,
              "\n Run Time Each Test %d seconds, end at %s \n",
                                                 runTime, timedate);
             testx = -2;
             saveEnd = FALSE;
       }
   }
   else
   {
       fprintf (outfile, "%9.2f Frames Per Second\n" ,
                                         resultS[0][firstTest]);
       fprintf (outfile, " First OpenGL Set Up %d ms, First Frame %d ms\n\n",
                                              setUpTicks, buildTicks);
       WhatConfig();
       fprintf(outfile, " %s\n", configData1);
       fprintf(outfile, " %s\n", configData2);
       fprintf(outfile, " %s\n\n", configData3);
                                   
       testx = -2;
   }
   
   fflush (outfile);
   fclose(outfile);
   resultsToSave = 0;
   doSave = FALSE;
   return TRUE;

} // saveData


