/* 
 *  VideoGL1 Copyright (C) Roy Longbottom 2004
 *
 *  This program allow screen size and colour settings to be
 *  selected and runs the benchmark OneGL1.exe at each setting.
 *  A Try fascility is provided, running the supplied program
 *  showsize.exe. At the end of all tests the supplied vesa.exe
 *  is run and some graphics type information added to log file
 *  VideoLog.txt. Buttons are provided to select and view
 *  OneGL1 results from VideoLog.
 *  
 *  Modules
 * 
 *  VideoGL1.rc - Menus and Dialog boxes AboutDlgProc(), 
 *                ResultSelect(), ResultData() and TestPanel()
 *
 *  VideoGL1.h, VideoGL1.ico
 *
 *  Uses OneGL1.exe, showsize.exe, vesa.exe
 *
 *  VideoGL1.cpp Functions
 *
 *  WinMain() - usual Windows functions and autoRun
 *  InitApp() - usual Windows functions, processes command line,
 *              allocates variable values, verifies monitor settings
 *  WindowProc() - usual Windows functions for msgs, buttons & menus
 *  AboutDlgProc() - displays Help and About
 *  CleanUpAndPostQuit() - Exit
 *  TestPanel() - Two text boxes INIT_SET and TEST_NOTES
 *                Two data boxes CAN_DO and CANNOT_DO
 *                4 colums x 9 rows of tick boxes for setting
 *                selection 500 +
 *                4 column clear and 4 column set buttons 601 - 608
 *                TRY_IT and TEST_IT buttons for testOrRun()
 *                SEE_RESULTS button for findLastLog() and ResultData()
 *                FIND_RESULTS button for GetFileName(), ResultSelect()
 *                and findLastLog()
 *                EXIT_BUTTON CleanUpAndPostQuit()
 *                MENU_ABOUT AboutDlgProc()
 *                MENU_RHELP AboutDlgProc() Description
 *                MENU_BHELP AboutDlgProc() Batch Parameters (InitApp())
 *                MENU_EXIT as EXIT_BUTTON 
 *                MENU_TEST as TEST_IT button
 *                MENU_TRY as TRY_IT button
 *                MENU_DISPLAY as SEE_RESULTS
 *                MENU_SELECT as FIND_RESULTS
 *  ResultSelect() - Displays list of results in log file and selection
 *  ResultData() - Displays selected results
 *  displayWhatever() - selects 1 of above 3 dialog boxes 
 *  customize1() - Benchmark specific variables and headings
 *  customize2() - Benchmark specific variables color bits/pixel
 *  findLastLog() - first reads log file to count number of entries
 *                  then runs readData() - Benchmark specific
 *  readData() - reads configuration data for all entries or a set of
 *               results - Benchmark specific 
 *  GetFileName() - calls Windows standard Open File dialog
 *  testOrRun() - Loops through selected screen settings and executes
 *                either OneGL1.exe or showsize.exe 
 *  addVesa() - executes vesa.exe on Exit
 */
 
#define NAME "SelectSettings"
#define TITLE "  Roy Longbottom Function Timing Test"
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <process.h>
#include "VideoGL1.h"


//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
void displayWhatever(HWND);
BOOL FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ResultData( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ResultSelect( HWND, UINT, WPARAM, LPARAM);
BOOL testOrRun();
BOOL findLastLog();
BOOL readData(int);
static BOOL GetFileName();
BOOL addVesa();
void customize1();
void customize2();

// ************* GLOBAL VARIABLES **********
 
char   thisProgram[16];
char   logFile[80] = "VideoLog.txt";
char   thisLog[256];
char   note1[100] = " This program changes screen settings to those chosen and runs ";
char   note2[100] = " Results are saved in log file ";
char   note3[100] = " The screen settings can be tried before running the tests";
char   note4[600] = " Commands ";
char   note5[100] = " ";

HWND    mainHWND;
HWND    testHwnd = NULL;
HWND    resultHwnd = NULL;
HWND    selectHwnd = NULL;
HINSTANCE  MyInstance;
DEVMODE devmode;
DEVMODE *devmodes;

short  keyPress;

char   version[30]; 
char   windowText1[256];
char   windowText2[256];
char   commandLine[400] = "";
char   currentDir[MAX_PATH];
char   searchDir[MAX_PATH];
char   startSettings[100];
char   runParameters[500];
char   heading1[10][8];
char   heading2[10][8];
char   config[4][100];
char   details[50][200];
char   bufd[200] = "Start";

BOOL   bQuit = FALSE;
BOOL   autoRun = FALSE;
BOOL   change = FALSE;
BOOL   colourChoice[9][4];
BOOL   canDoChoice[9][4];
BOOL   runProgram = TRUE;
BOOL   bitCommand = FALSE;
BOOL   b8  = FALSE;
BOOL   b16 = FALSE;
BOOL   b24 = FALSE;
BOOL   b32 = FALSE;
BOOL   useTestPanel = TRUE;
BOOL   useLast = TRUE;
BOOL   showIndex = FALSE;
BOOL   runDone = FALSE;
BOOL   doVesa = TRUE;
BOOL   doAllSizes = FALSE;

DWORD  xResolution[9];
DWORD  yResolution[9];
DWORD  colorBits[4];
int    screenHz[9][4];
int    useDev[9][4];
int    maxRes = 9;
int    maxUseRes = 3;
int    totalTests = 0;
int    num_modes = 0;
int    wideStart;
int    highStart;
int    bppStart;
int    freqStart;
int    resultsIn;
int    runningTest = 0;
int    wideUsed[40];
int    highUsed[40];
int    bitsUsed[40];
int    resCount = 0;
int    theseSpeeds;

double  results[40][10];


void displayWhatever(HWND hwnd)
{
    SetWindowText(mainHWND, windowText1);        
        
    if (testHwnd)   DestroyWindow(testHwnd);
    if (resultHwnd) DestroyWindow(resultHwnd);
    if (selectHwnd) DestroyWindow(selectHwnd);
    if (useTestPanel)
    {
        testHwnd = CreateDialog( MyInstance,"DIALOG_8", hwnd, (DLGPROC)TestPanel);  
    }
    else
    {
        if (showIndex)
        {
            selectHwnd = CreateDialog( MyInstance,"DIALOG_10", hwnd, (DLGPROC)ResultSelect);
        }
        else
        {
            resultHwnd = CreateDialog( MyInstance,"DIALOG_9", hwnd, (DLGPROC)ResultData);
        }
    }
} // displayWhatever


// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    if (autoRun)
    {
        displayWhatever(mainHWND);
        if (testHwnd)   DestroyWindow(testHwnd);                                                                            
        testOrRun();
        if (doVesa) addVesa();
        displayWhatever(mainHWND);
        CleanUpAndPostQuit();
    }        
    while(!bQuit)
    {
        if (change)
        {
            change = FALSE;
            displayWhatever(mainHWND);
        }
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }
    return msg.wParam;
    
} // WinMain

// ***************************************************************************
//                    Initialization and object creation                     *
// ***************************************************************************

// InitApp
// Creates window

static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    int  i, j, m;
    int  maxHz;
    int  theseRes;
    HDC  hdc;
  
    // possible resolutions

    xResolution[0] =  640;
    yResolution[0] =  480;
    
    xResolution[1] =  800;
    yResolution[1] =  600;
    
    xResolution[2] = 1024;
    yResolution[2] =  768;
    
    xResolution[3] = 1152;
    yResolution[3] =  864;
    
    xResolution[4] = 1280;
    yResolution[4] = 1024;
    
    xResolution[5] = 1600;
    yResolution[5] = 1200;
    
    xResolution[6] = 1800;  // 1.25
    yResolution[6] = 1440;
    
    xResolution[7] = 1920; 
    yResolution[7] = 1440;
    
    xResolution[8] = 2048;
    yResolution[8] = 1536;
      
    colorBits[0] = 8;
    colorBits[1] = 16;
    colorBits[2] = 24;
    colorBits[3] = 32;

    // initial resolutions not to use and to use
    
    for(i=0; i<maxRes; i++)
    {
         for(j=0; j<4; j++)
         {
              colourChoice[i][j] = FALSE;
              canDoChoice[i][j] = FALSE;
         }
    }
     
    customize1();

    // set up and registers the window class
    
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    // Create the window

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU,
            0,
            0,
            640,
            480,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));
    if (strlen(commandLine) > 1)
    {
        strcat(note4, commandLine);
    }
    else
    {
        strcpy(note4, "");
    }

    GetCurrentDirectory(MAX_PATH, currentDir);
    strcpy(searchDir, currentDir);
 
    autoRun = FALSE;
       
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    num_modes = 0;
    while (EnumDisplaySettings(NULL, num_modes, &devmode))
        num_modes++;

    // fill an array with all the devmodes so we don't have to keep
    //   grabbing 'em.
    
    devmodes = (DEVMODE*)malloc(sizeof(DEVMODE)*num_modes);
    num_modes = 0;
    while (EnumDisplaySettings(NULL, num_modes, &devmodes[num_modes]))
        num_modes++;

    hdc = GetDC(hwnd);
    bppStart = GetDeviceCaps(hdc, BITSPIXEL);
    wideStart = GetDeviceCaps(hdc, HORZRES);    
    highStart = GetDeviceCaps(hdc, VERTRES);
    freqStart = GetDeviceCaps(hdc, VREFRESH);
    ReleaseDC(hwnd, hdc);

    for(i=0; i<maxRes; i++)
    {
         for(j=0; j<4; j++)
          {
              maxHz = 0;   
              screenHz[i][j] = freqStart;
              for (m=0; m<num_modes; m++)
              {
                  if (devmodes[m].dmPelsWidth  == xResolution[i] &&
                      devmodes[m].dmPelsHeight == yResolution[i] &&
                      devmodes[m].dmBitsPerPel == colorBits[j])
                   { 
                      canDoChoice[i][j] = TRUE;
                      useDev[i][j] = m;
                      if ((int)devmodes[m].dmDisplayFrequency > maxHz)
                             maxHz = devmodes[m].dmDisplayFrequency;
                   }
              }
              if (maxHz < freqStart) screenHz[i][j] = maxHz; 
          }
    }
    for(i=0; i<maxUseRes; i++)
    {
         for(j=0; j<4; j++)
         {
              if (!canDoChoice[i][j]) colourChoice[i][j] = FALSE;
         }
    }

    if (freqStart > 1)
    {
        sprintf(startSettings, " Initial screen setting %d x %d x %d, at %d Hz",
                  wideStart, highStart, bppStart, freqStart);
    }
    else
    {
        sprintf(startSettings, " Initial screen setting %d x %d x %d, at default Hz",
                  wideStart, highStart, bppStart);
    }
    strcat(note1, thisProgram);
    strcat(note2, logFile);

    // command line scan

    if (strlen(commandLine))
    {
       for (j=0; j<lstrlen(commandLine); j++)
       {
          if (strnicmp(&commandLine[j],  "AUT", 3) == 0)
          {
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "NOV", 3) == 0)
          {
              doVesa = FALSE; // ccccc
          }
          if (strnicmp(&commandLine[j],  "MAX", 3) == 0)
          {
              doAllSizes = TRUE;
          }
          if (strnicmp(&commandLine[j],  "B8", 2) == 0)
          {
              b8 = TRUE;
              bitCommand = TRUE;
          }
          if (strnicmp(&commandLine[j],  "B16", 3) == 0)
          {
              b16 = TRUE;
              bitCommand = TRUE;
          }
          if (strnicmp(&commandLine[j],  "B24", 3) == 0)
          {
              b24 = TRUE;
              bitCommand = TRUE;
          }
          if (strnicmp(&commandLine[j],  "B32", 3) == 0)
          {
              b32 = TRUE;
              bitCommand = TRUE;
          }
       }
    }
    if (bitCommand)
    {
        theseRes = maxUseRes;
        if (doAllSizes) theseRes = maxRes;
        for(i=0; i<maxRes; i++)
        {
             for(j=0; j<4; j++)
             {
                 colourChoice[i][j] = FALSE;
             }
        }     
        for(i=0; i<theseRes; i++)
        {
            if (b8  && canDoChoice[i][0]) colourChoice[i][0] = TRUE;
            if (b16 && canDoChoice[i][1]) colourChoice[i][1] = TRUE;
            if (b24 && canDoChoice[i][2]) colourChoice[i][2] = TRUE;
            if (b32 && canDoChoice[i][3]) colourChoice[i][3] = TRUE;
        }
    }
    else
    {
        if (doAllSizes)
        {
            customize2();
        }
    }
    
    // Display the window
    
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);
    sprintf(windowText2, " Try different screen settings - Window size 640 x 480");

    displayWhatever(mainHWND);
    
    return TRUE;
    
}  // InitApp



// WindowProc - handle messages for the main application window


long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{          
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;

        case MENU_RHELP:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"RelHelp", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;

        case MENU_BHELP:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"BatchHelp", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;
                      
       case MENU_TEST:
            if (totalTests == 0)
            {
                 MessageBox(NULL, "No setting selected", "Error", MB_ICONWARNING | MB_OK);
            }
            else
            {
                runProgram = TRUE;
                if (testHwnd)   DestroyWindow(testHwnd);                                                                            
                if (!testOrRun())
                {
                    CleanUpAndPostQuit();
                    return 0;
                }
                runDone = TRUE;
            }
            displayWhatever(mainHWND);
            break;

        case MENU_TRY:
            SetWindowText(mainHWND, windowText2);        
            if (totalTests == 0)
            {
                 MessageBox(NULL, "No setting selected", "Error", MB_ICONWARNING | MB_OK);
            }
            else
            {
                runProgram = FALSE;
                if (testHwnd)   DestroyWindow(testHwnd);                                                                            
                if (!testOrRun())
                {
                    CleanUpAndPostQuit();
                    return 0;
                }
            }
            displayWhatever(mainHWND);
            break;

        case MENU_DISPLAY:
            strcpy(thisLog, logFile);
            SetCurrentDirectory(currentDir);
            useTestPanel = FALSE;
            useLast = TRUE;
            showIndex = FALSE;
            if (!findLastLog()) useTestPanel = TRUE;
            displayWhatever(mainHWND);                                      
            break;
        
        case MENU_SELECT:
            strcpy(thisLog, logFile);
            SetCurrentDirectory(searchDir);
            if  (GetFileName())
            {                
                GetCurrentDirectory(MAX_PATH, searchDir);
                useTestPanel = FALSE;
                showIndex = TRUE;
                useLast = FALSE;
                if (!findLastLog()) useTestPanel = TRUE;
            }
            SetCurrentDirectory(currentDir);
            displayWhatever(mainHWND);                                      
            break;

        case MENU_EXIT:
            if (runDone && doVesa) addVesa();
            CleanUpAndPostQuit();
            break;
                
        }
        break;
   
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc




// AboutDlgProc - processes messages for the about dialog.

BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} // AboutDlgProc




void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    hdlg = hdlg;
    int i, j, n;
    int FAR  tabs[11] = {23, 46, 64, 80, 100, 270, 300, 330, 360, 370, 380};
    HWND     hwndName;
    char buf[80];

     switch( msg )
     {
       case WM_INITDIALOG:
            SendDlgItemMessage(hdlg, CAN_DO, LB_SETTABSTOPS, (WPARAM) 5, (LPARAM) (int FAR*) tabs);
            SendDlgItemMessage(hdlg, CANNOT_DO, LB_SETTABSTOPS, (WPARAM) 5, (LPARAM) (int FAR*) tabs);
            hwndName = GetDlgItem(hdlg, TEST_NOTES);    
            if (strlen(note4) > 1) SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note4); 
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note1); 
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note2); 
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note3); 

            n = 1;
            totalTests = 0;
            for(i=0; i<maxRes; i++)
            {
               for(j=0; j<4; j++)
               {
                  if (colourChoice[i][j])
                  {
                       CheckDlgButton(hdlg, 500 + n, BST_CHECKED);
                       totalTests = totalTests + 1;
                  }
                  n = n + 1;        
               }
            }
            for(j=0; j<4; j++)
            {
                 for(i=0; i<maxRes; i++)
                 {
                      sprintf(buf, " %4d%cx %4d%cx %2d%c", xResolution[i], 9, yResolution[i], 9, colorBits[j], 9);
                      if (canDoChoice[i][j])
                      {
                          if (colourChoice[i][j]) strcat(buf, "**");
                          hwndName = GetDlgItem(hdlg, CAN_DO);    
                          SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) buf);
                      }
                      else
                      {
                          hwndName = GetDlgItem(hdlg, CANNOT_DO);    
                          SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) buf);
                      }
                 }
            }
            hwndName = GetDlgItem(hdlg, INIT_SET);    
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) startSettings);
            return( TRUE );
                    
        case WM_COMMAND:

        switch( LOWORD( wparam ) )
        {
            case 601:
              for(i=0; i<maxRes; i++)
              {
                 CheckDlgButton(hdlg, 501+i*4, BST_UNCHECKED);
              }
              break;
            
            case 602:
              for(i=0; i<maxRes; i++)
              {
                 CheckDlgButton(hdlg, 502+i*4, BST_UNCHECKED);
              }
              break;
    
            case 603:
              for(i=0; i<maxRes; i++)
              {
                 CheckDlgButton(hdlg, 503+i*4, BST_UNCHECKED);
              }
              break;
    
            case 604:
              for(i=0; i<maxRes; i++)
              {
                 CheckDlgButton(hdlg, 504+i*4, BST_UNCHECKED);
              }
              break;
    
            case 605:
              for(i=0; i<maxRes; i++)
              {
                 if (canDoChoice[i][0])
                 CheckDlgButton(hdlg, 501+i*4, BST_CHECKED);
              }
              break;
    
            case 606:
              for(i=0; i<maxRes; i++)
              {
                 if (canDoChoice[i][1])
                 CheckDlgButton(hdlg, 502+i*4, BST_CHECKED);
              }
              break;
    
            case 607:
              for(i=0; i<maxRes; i++)
              {
                 if (canDoChoice[i][2])
                 CheckDlgButton(hdlg, 503+i*4, BST_CHECKED);
              }
              break;
              
            case 608:
              for(i=0; i<maxRes; i++)
              {
                 if (canDoChoice[i][3])
                 CheckDlgButton(hdlg, 504+i*4, BST_CHECKED);
              }
              break;
        }


        if( HIWORD( wparam ) == BN_CLICKED)
        {
              n = 1;
              for(i=0; i<maxRes; i++)
              {
                 for(j=0; j<4; j++)
                 {
                    if (!canDoChoice[i][j])
                    {
                        CheckDlgButton(hdlg, 500+n, BST_UNCHECKED);
                    }
                    n = n + 1; 
                 }
              }
              n = 1;
              totalTests = 0;
              for(i=0; i<maxRes; i++)
              {
                 for(j=0; j<4; j++)
                 {
                    if (IsDlgButtonChecked(hdlg, 500+n) == BST_CHECKED)
                    {
                       colourChoice[i][j] = TRUE;
                       totalTests = totalTests + 1;
                    }
                    else
                    {
                       colourChoice[i][j] = FALSE;
                    }
                    n = n + 1; 
                 }
              }
            hwndName = GetDlgItem(hdlg, CAN_DO);    
            SendMessage(hwndName, LB_RESETCONTENT, 0, 0);
            hwndName = GetDlgItem(hdlg, CANNOT_DO);    
            SendMessage(hwndName, LB_RESETCONTENT, 0, 0);
            for(j=0; j<4; j++)
            {
                 for(i=0; i<maxRes; i++)
                 {
                      sprintf(buf, " %4d%cx %4d%cx %2d%c", xResolution[i], 9, yResolution[i], 9, colorBits[j], 9);
                      if (canDoChoice[i][j])
                      {
                          if (colourChoice[i][j]) strcat(buf, "**");
                          hwndName = GetDlgItem(hdlg, CAN_DO);    
                          SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) buf);
                      }
                      else
                      {
                          hwndName = GetDlgItem(hdlg, CANNOT_DO);    
                          SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) buf);
                      }
                 }
            }

            switch (LOWORD (wparam))
            {
                case TEST_IT:
                  PostMessage(mainHWND, WM_COMMAND, MENU_TEST, 0);
                 break;
                 
                case TRY_IT:
                  PostMessage(mainHWND, WM_COMMAND, MENU_TRY, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;

               case SEE_RESULTS:
                 PostMessage(mainHWND, WM_COMMAND, MENU_DISPLAY, 0);
                 break;

               case FIND_RESULTS:
                 PostMessage(mainHWND, WM_COMMAND, MENU_SELECT, 0);
                 break;
            }
        }
        return( TRUE );         
     }
     return( FALSE );
    
} // TestPanel


BOOL testOrRun()
{
    int i, j, m;
    long displayChange;
    int  freq;
    BOOL haveChanged = FALSE;
    
    runningTest = 0;
    for(j=0; j<4; j++)
    {
         for(i=0; i<maxRes; i++)
         {
              if (canDoChoice[i][j] && colourChoice[i][j])
              {
                  m = useDev[i][j];
                  freq = screenHz[i][j];
                  if (freq > 1)
                  {
                      sprintf(bufd, " %4d x %4d x %2d, frequency %d Hz", xResolution[i], yResolution[i], colorBits[j], freq);
                  }
                  else
                  {
                      sprintf(bufd, " %4d x %4d x %2d", xResolution[i], yResolution[i], colorBits[j]);
                  }
                  devmode = devmodes[m];
                  devmode.dmDisplayFrequency = freq; 
                  
                  displayChange = ChangeDisplaySettings(&devmode, CDS_TEST);        
                  if (displayChange == DISP_CHANGE_RESTART)
                  {
                      if (!autoRun) MessageBox( NULL, bufd, "Cannot Set Needs Re-Boot", MB_ICONWARNING | MB_OK );
                      canDoChoice[i][j] = FALSE;
                  }
                  else if (displayChange != DISP_CHANGE_SUCCESSFUL)
                  {
                      if (!autoRun) MessageBox( NULL, bufd, "Cannot Set Choice", MB_ICONWARNING | MB_OK );
                      canDoChoice[i][j] = FALSE;
                  }
                  else
                  {
                       if (runProgram)
                       {
                           displayChange = ChangeDisplaySettings(&devmode, CDS_UPDATEREGISTRY);
                           if (displayChange != DISP_CHANGE_SUCCESSFUL)
                           {
                               
                                MessageBox( NULL, bufd, "Cannot Set Choice", MB_ICONWARNING | MB_OK );
                                canDoChoice[i][j] = FALSE;
                           }
                           else
                           {
                               runningTest = runningTest + 1;
                               haveChanged = TRUE;
                               if (totalTests == 1)
                               {
                                   sprintf(runParameters, " RUN, ");
                               }
                               else
                               {
                                   if (runningTest == 1)
                                   {
                                       sprintf(runParameters, " SAVEHR, ");
                                   }
                                   else if (runningTest == totalTests)
                                   {
                                       sprintf(runParameters, " SAVERE, ");
                                   }
                                   else
                                   {
                                       sprintf(runParameters, " SAVER, ");
                                   }
                               }
                               strcat(runParameters, commandLine);
                               spawnl(P_WAIT, thisProgram, thisProgram, runParameters, NULL); // system
                               if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                               {
                                   i = maxRes;
                                   j = 4;
                                   doVesa = FALSE;
                               }
                           }
                       }
                       else
                       {
                           displayChange = ChangeDisplaySettings(&devmode, 0);
                           if (displayChange != DISP_CHANGE_SUCCESSFUL)
                           {               
                                MessageBox( NULL, bufd, "Cannot Set Choice", MB_ICONWARNING | MB_OK );
                                canDoChoice[i][j] = FALSE;
                           }
                           else
                           {
                               spawnl(P_WAIT, "showsize.exe", NULL); // system
                               if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                               {
                                   i = maxRes;
                                   j = 4;
                               }
                               Sleep(500);
                                                             
                               //  Change back
                               
                               sprintf(bufd, " Back to normal");
                               displayChange = ChangeDisplaySettings(NULL, 0);
                               if (displayChange != DISP_CHANGE_SUCCESSFUL)
                               {               
                                   MessageBox( NULL, bufd, "Cannot Set Choice To Original", MB_ICONWARNING | MB_OK );
                               }
                               GdiFlush();
                           }
                       }
                  }
              }
         }
    }
    if (haveChanged)
    {
        devmode.dmPelsWidth  = wideStart;
        devmode.dmPelsHeight = highStart;
        devmode.dmBitsPerPel = bppStart;
        devmode.dmDisplayFrequency = freqStart;
        displayChange = ChangeDisplaySettings(&devmode, CDS_UPDATEREGISTRY);
        if (displayChange != DISP_CHANGE_SUCCESSFUL)
        {
            MessageBox( NULL, "Cannot Restore Screen Resolution", "Error", MB_OK );
        }
    }
    ShowWindow(mainHWND, SW_SHOWDEFAULT);

    return TRUE;

} // testOrRun


BOOL FAR PASCAL ResultData( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
   int i, k;
   char buf[200];
   char buf2[260];

   hdlg = hdlg;
   wparam = wparam;
   lparam = lparam;

   switch( msg )
   {
       case WM_INITDIALOG:
        sprintf (buf2, "Screen");
        for (i=0; i<theseSpeeds; i++)
        {
            sprintf (buf, "%c%8s", 9, heading1[i]);
            strcat(buf2, buf);
        }
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        sprintf (buf2, "    ");
        for (i=0; i<theseSpeeds; i++)
        {
            sprintf (buf, "%c%8s", 9, heading2[i]);
            strcat(buf2, buf);
        }
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        for (i = 0; i<resultsIn; i++)
        {
           sprintf(buf, "%4d %2d%c", wideUsed[i], bitsUsed[i], 9);
           for (k=0; k<theseSpeeds; k++)
           {
               sprintf (buf2, "%9.2f%c", results[i][k], 9);
               strcat(buf, buf2); 
           }
           SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf);
        }
        sprintf (buf2, "    ");
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        for (i=0; i<4; i++)
        {
            SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) config[i]);
        }
        return( TRUE );         
     
       case WM_COMMAND:
          if( HIWORD( wparam ) == BN_CLICKED)
          {
              switch (LOWORD (wparam))
              {                 
                  case IDOK:
                     DestroyWindow(resultHwnd);
                     useTestPanel = TRUE;
                     change = TRUE;
                     break;
              }
          }
          return TRUE;      
   }
   return FALSE;
   
} // ResultData

BOOL FAR PASCAL ResultSelect( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
   int i;
   int FAR  tabs[4] = {80, 260, 270, 280};
   int index = 0;


   hdlg = hdlg;
   wparam = wparam;
   lparam = lparam;

   switch( msg )
   {
       case WM_INITDIALOG:
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETTABSTOPS, (WPARAM) 4, (LPARAM) (int FAR*) tabs);
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETHORIZONTALEXTENT, 1200, (LPARAM)NULL);
        for (i = 1; i<resCount+1; i++)
        {
           SendDlgItemMessage(hdlg, ENTRY_BOX, LB_ADDSTRING, 0, (LPARAM) details[i]);
        }
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETCURSEL, 0, 0L);
        return( TRUE );         
     
       case WM_COMMAND:
          if( HIWORD( wparam ) == BN_CLICKED)
          {
              switch (LOWORD (wparam))
              {                 
                  case IDOK:
                     index = (int)SendDlgItemMessage(hdlg, ENTRY_BOX, LB_GETCURSEL, 0, 0);
                     DestroyWindow(selectHwnd);
                     useTestPanel = FALSE;
                     showIndex = FALSE;
                     if (!readData(index+1)) useTestPanel = TRUE;
                     change = TRUE;
                     break;

                  case IDCANCEL:
                     DestroyWindow(selectHwnd);
                     useTestPanel = TRUE;
                     change = TRUE;
                     break;
              }
          }
          return TRUE;      
   }
   return FALSE;
   
} // ResultSelect


static BOOL GetFileName()
{
    static char    filterList1[] = "File (*.txt)" \
                                        "\0" \
                                        "*.txt" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList1;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = thisLog;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;

    rc = GetOpenFileName( &of );
    strcpy(thisLog, of.lpstrFile);
   
    return( rc );
    
} // GetFileName


BOOL addVesa()
{
    FILE *infile;
    FILE *outfile;
    char buf2[100];
    char buffer[100];

    // What VESA

    system("vesa.exe");
    
    outfile = fopen(logFile, "a+");
    if (outfile == NULL)
    {
          MessageBox(mainHWND, "Cannot open log file", logFile,
                             MB_ICONINFORMATION | MB_OK);
          return FALSE;
    }
    if (strlen(note4) > 1) fprintf(outfile, " %s\n\n", note4);
    infile = fopen("whatvesa.txt", "r");
    if (infile == NULL)
    {
          MessageBox(mainHWND, "Cannot open file whatvesa.txt", "Error",
                             MB_ICONINFORMATION | MB_OK);
          fclose(outfile);
          return FALSE;
    }
    while( fgets( buffer, 80, infile ) != NULL )
    {
        lstrcpyn(buf2, buffer, lstrlen(buffer));
        fprintf (outfile, " %s\n", buf2);
    }
    fclose(infile);
    fclose(outfile);
    return TRUE;
    
} // addVesa


// Following are benchmark specific

void customize1()
{
    int i;
    
    for(i=0; i<maxUseRes; i++)
    {
        colourChoice[i][1] = TRUE;
        colourChoice[i][3] = TRUE;
    }

    strcpy(thisProgram, "OneGL1.exe");
    strcpy(version, "Version 2.4");
    strcpy (windowText1, "  Roy Longbottom's OpenGL Function Timing Test");

    theseSpeeds = 8;

    strcpy(heading1[0], " Single");
    strcpy(heading2[0], "  Plain");
    strcpy(heading1[1], " Single");
    strcpy(heading2[1], "Texture");
    strcpy(heading1[2], "  Multi");
    strcpy(heading2[2], "  Plain");
    strcpy(heading1[3], "  Multi");
    strcpy(heading2[3], "Texture");
    strcpy(heading1[4], "   More");
    strcpy(heading2[4], "Texture");
    strcpy(heading1[5], "  More+");
    strcpy(heading2[5], "Texture");
    strcpy(heading1[6], "   Wire");
    strcpy(heading2[6], "Kitchen");
    strcpy(heading1[7], "  Tiled");
    strcpy(heading2[7], "Kitchen");
}

void customize2()
{
    int i;
    
    for(i=0; i<maxRes; i++)
    {
        if (canDoChoice[i][1]) colourChoice[i][1] = TRUE;
        if (canDoChoice[i][3]) colourChoice[i][3] = TRUE;
    }
}

BOOL readData(int resCount2)
{
   FILE *infile;
   char buf[350];
   char b[11][10];
   int  i, j, c;
   int  newCount = 0;
   BOOL fin = FALSE;
   
   for (i=0; i<40; i++)
   {
       wideUsed[i] = 0;
       highUsed[i] = 0;
       bitsUsed[i] = 0;
       for (j=0; j<10; j++)
       {
           results[i][j] = 0;
       }
   }
   sprintf(config[0], "  OS?");
   sprintf(config[1], "  CPU?");
   sprintf(config[2], "  Memory?");
   sprintf(config[3], "  end time?");

   infile = fopen(thisLog,"r");
   if (infile == NULL)
   {
       MessageBox(NULL, "Cannot open log file", thisLog, MB_ICONWARNING | MB_OK);
       return FALSE;
   }
   while (newCount < resCount2 && fgets(buf, 120, infile) != NULL)
   {
       if (strnicmp(buf, "        Windows 95/NT OpenGL", 28) == 0)
       {
           newCount = newCount + 1;
       }   
       else if (strnicmp(buf, "       Windows 95/NT/XP Open", 28) == 0)   
       {
           newCount = newCount + 1;
       }
   }
   c = 0;
   resultsIn = 0;
   if (resCount2 > 0)
   {
       while (!fin && fgets(buf, 120, infile) != NULL)
       {
           c = c + 1;
           if (c > 8)
           {
               if (strlen(buf) < 86)
               {
                   fin = TRUE;
               }
               else
               {
                   sscanf(buf, "%5c%5c%4c%9c%9c%9c%9c%9c%9c%9c%9c",
                                b[0],  b[1],  b[2],  b[3],  b[4],  b[5],
                                b[6],  b[7],  b[8],  b[9],  b[10]);
                   wideUsed[resultsIn] = atoi(b[0]);
                   highUsed[resultsIn] = atoi(b[1]);
                   bitsUsed[resultsIn] = atoi(b[2]);
                    
                   for (i=0; i< 8; i++)
                   {
                       results[resultsIn][i] = strtod(b[i+3], NULL);
                   }
                   resultsIn = resultsIn + 1;
               } 
           }
       }
       c = 0;
       fin = FALSE;
       while (!fin && fgets(buf, 120, infile) != NULL)
       {
           c = c + 1;
           if (strlen(buf) > 1)
           {
               if (c == 5 && strnicmp(buf, "  Win", 5) == 0)
               {
                   strcpy (config[0], buf);
                   config[0][strlen(buf)-1] = NULL;
               }
               if (c == 6 && strnicmp(buf, "  CPU", 5) == 0)
               {
                   strcpy (config[1], buf);
                   config[1][strlen(buf)-1] = NULL;
               }
               if (c == 7 && strnicmp(buf, "  Mem", 5) == 0)
               {
                   strncpy (config[2], buf, strlen(buf)-1);
                   config[2][strlen(buf)-1] = NULL;
               }
               if (c == 9 && strnicmp(buf, " Run ", 5) == 0)
               {
                   c = 0;
                   for (i=0; i<lstrlen(buf)-1; i++)
                   {
                       if (buf[i] == 44)
                       {
                           c = i + 8;
                       }
                   }
                   j = 1;
                   for (i=c; i<lstrlen(buf)-1; i++)
                   {
                       config[3][j] = buf[i];
                       j = j + 1;
                   }
                   config[3][j] = NULL;                   
                   fin = TRUE;
               }
           }
       }
       if (!useLast && resCount2 < 51)
       {
           if (strlen(details[resCount2]) + strlen(config[2]) < 200)
           {
               sprintf(details[resCount2], " %s%c%s%c%s", config[3], 9, config[0], 9, config[1]);
           }
           else
           {
               sprintf(details[resCount2], " %s%c%s%c  CPU?", config[3], 9, config[0], 9);
           }
       }                  
   }
   fclose(infile);

   return TRUE;

} // readData 



BOOL findLastLog()
{
   FILE *infile;
   char buf[350];
   int  i;

   resCount = 0;
   infile = fopen(thisLog,"r");
   if (infile == NULL)
   {
       MessageBox(NULL, "Cannot open log file", thisLog, MB_ICONWARNING | MB_OK);
       return FALSE;
   }
   while (fgets(buf, 120, infile) != NULL)
   {
       if (strnicmp(buf, "        Windows 95/NT OpenGL", 28) == 0)
       {
           resCount = resCount + 1;
       }
       else if (strnicmp(buf, "       Windows 95/NT/XP Open", 28) == 0)   
       {
           resCount = resCount + 1;
       }
   }
   fclose(infile);
   if (useLast)
   {
       readData(resCount);
   }
   else
   {
        for (i = 0; i<resCount+1; i++)
        {
             readData(i);
        }
   }
   return TRUE;
   
} // findLastLog


